/* eslint-disable prettier/prettier */
import http from 'http';
import formidable from 'formidable';
import fs from 'fs';
import path from 'path';

// Создаем директорию для загрузок, если она не существует
const __dirname = path.resolve();
const uploadDir = path.join(__dirname, 'uploads');
if (!fs.existsSync(uploadDir)) {
  fs.mkdirSync(uploadDir);
}

// Создаем сервер
const server = http.createServer((req, res) => {
  // Добавляем заголовки CORS
  res.setHeader('Access-Control-Allow-Origin', '*');
  res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
  res.setHeader('Access-Control-Allow-Headers', 'Content-Type');

  if (req.method === 'OPTIONS') {
    res.writeHead(204);
    res.end();
    return;
  }

  if (req.method === 'POST') {
    const form = formidable({
      uploadDir: uploadDir,
      keepExtensions: true,
      multiples: true // разрешаем множественную загрузку файлов
    });

    form.parse(req, (err, fields, files) => {
      if (err) {
        console.error('Error parsing the form:', err);
        res.writeHead(500, { 'Content-Type': 'application/json' });
        res.end(JSON.stringify({ error: 'Error parsing the form' }));
        return;
      }

   // console.log('Files:', files); // Отладочная информация

      const uploadedFiles = files.file; // Получаем массив файлов
      if (!uploadedFiles || uploadedFiles.length === 0) {
        res.writeHead(400, { 'Content-Type': 'application/json' });
        res.end(JSON.stringify({ error: 'No file uploaded' }));
        return;
      }
    // console.log('File:', uploadedFiles[0]);
      const file = Array.isArray(uploadedFiles) ? uploadedFiles[0] : uploadedFiles; // Берем первый файл из массива или единственный файл
      const oldPath = file.filepath;
      const newPath = path.join(uploadDir, file.originalFilename);

      console.log(file);

      // Перемещение файла в нужную директорию
      fs.rename(oldPath, newPath, (err) => {
        if (err) {
          console.error('Error saving the file:', err);
          res.writeHead(500, { 'Content-Type': 'application/json' });
          res.end(JSON.stringify({ error: 'Error saving the file' }));
          return;
        }

        res.writeHead(200, { 'Content-Type': 'application/json' });
        res.end(JSON.stringify({ message: 'File uploaded successfully' }));
      });
    });
  } else {
    res.writeHead(404, { 'Content-Type': 'text/plain' });
    res.end('Not Found');
  }
});

// Сервер будет слушать запросы на порту 3000
const PORT = 4000;
server.listen(PORT, () => {
  console.log(`Server is running on http://localhost:${PORT}/`);
});